/* ----------------------------------------------------------------------------
 * defkeys.h
 * emulates middle click and right click
 *
 * Copyright 2003, 2004 Colin Leroy (colin@colino.net).
 *
 * Many thanks to uinput author, <aris@cathedrallabs.org>, for his
 * really useful help.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------*/
#ifndef __DEFKEYS_H__
#define __DEFKEYS_H__

#ifdef __powerpc__

#define BUTTON2MOD	0
#define BUTTON2KEY	KEY_F10
#define BUTTON3MOD	0
#define BUTTON3KEY	KEY_F11

#else

#define BUTTON2MOD	0
#define BUTTON2KEY	0
#define BUTTON3MOD	0
#define BUTTON3KEY	0

#endif

#define SCROLLMOD	KEY_LEFTALT


#endif 
